// CRPlugin.cpp : Defines the entry point for the DLL application.
//

#import  "CRPluginWorker.tlb"
#include "CRPlugin.h"

#define  PLUGINAPI_VERSION    1
#define  PLUGINAPI_GUID       "C7037CD8-EA28-43af-ADC5-4BB00DD99FED"
#define  PLUGINAPI_TYPE       "Crystal Reports"
#define  PLUGINAPI_DESC       "Crystal Reports plugin for TestTrack Pro and Surround SCM"

#define  PLUGIN_GUID_LEN            39
#define  PLUGIN_TYPE_LEN            257
#define  PLUGIN_DESC_LEN            8193
#define  PLUGIN_ERR_LEN             8193
#define  PLUGIN_OUTFILE_LEN         4097

#define PLUGIN_MESG1             "Error initializing plugin library."
#define PLUGIN_MESG2             "Error while getting list of reports."
#define PLUGIN_MESG3             "Error while try to run the given report.";
#define PLUGIN_MESG4             "Error while try to execute the given command.";

#ifdef WIN32
typedef unsigned int uint32_t;
#include "stdafx.h"
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
    return TRUE;
}
#else
#include <stdint.h>
#endif

#include <cstring>
#include <string>
#include <exception>

using std::strcpy;

//----------------------------------------------------------------------------------------------------------
// Function: ToUtf8
//   Convert a wide character string to utf8 string
//
// Parameters:
//   widestring - wide character string to convert
//
// Returns:
//   char*  - pointer to the new utf8 string, caller is responsible for clean up
//----------------------------------------------------------------------------------------------------------
std::string ToUtf8(const std::wstring& widestring)
{
   char* pStrResult = NULL;
   std::string strResult;
   int nWStrSize = WideCharToMultiByte(CP_UTF8, 0, widestring.c_str(), -1, NULL, 0, NULL, NULL);

   if(nWStrSize > 0)
   {
      pStrResult = new char[nWStrSize + 1];
      memset(pStrResult, 0, nWStrSize + 1);
      int convresult = WideCharToMultiByte(CP_UTF8, 0, widestring.c_str(), -1, pStrResult, nWStrSize, NULL, NULL);
   }

   if(pStrResult)
   {
      // Assign pStrResult to string object which will make a copy of pStrResult and will take ownership
      // to delete it when it goes out of scope
      strResult = pStrResult;
      delete [] pStrResult;
      pStrResult = NULL;
   }

   return strResult;
}

//----------------------------------------------------------------------------------------------------------
// Function: Utf8ToWide
//   Convert a utf8 string to wide character format
//
// Parameters:
//   utf8Str - utf8 string to convert
//
// Returns:
//   wchar_t*  - pointer to the new wide format string, caller is responsible for clean up
//----------------------------------------------------------------------------------------------------------
static wchar_t* Utf8ToWide(const std::string utf8Str)
{
   int wideLen = MultiByteToWideChar(CP_UTF8, 0, utf8Str.c_str(), -1, NULL, 0);
   wchar_t *pWideStr = NULL;

   if(wideLen > 0)
   {
      pWideStr = new wchar_t[wideLen + 1];
      memset(pWideStr, 0, (wideLen + 1) * sizeof(wchar_t));

      MultiByteToWideChar(CP_UTF8, 0, utf8Str.c_str(), -1, pWideStr, wideLen);
   }
   else
   {
      pWideStr = new wchar_t[1];
      pWideStr[0] = L'\0';
   }

   return pWideStr;
}

// Functions to learn about the plug-in
//----------------------------------------------------------------------------------------------------------
// Function: GetPluginVersion
//   Interface that returns plugin version
//
// Parameters:
//   None
//
// Returns:
//   uint32_t  - Plugin Version no.
//----------------------------------------------------------------------------------------------------------
uint32_t GetPluginVersion()
{
   return PLUGINAPI_VERSION;
}

//----------------------------------------------------------------------------------------------------------
// Function: GetGUID
//   Interface to get plugin GUID
//
// Parameters:
//   char* pGUID - Plugin GUID
//
// Returns:
//   None
//----------------------------------------------------------------------------------------------------------
void GetGUID(char* pGUID)
{
   if(pGUID)
      strncpy(pGUID, PLUGINAPI_GUID, PLUGIN_GUID_LEN);
}

//----------------------------------------------------------------------------------------------------------
// Function: GetType
//   Interface to get plugin type
//
// Parameters:
//   char* pType - Plugin Type
//
// Returns:
//   None
//----------------------------------------------------------------------------------------------------------
void GetType(char* pType)
{
   if(pType)
      strncpy(pType, PLUGINAPI_TYPE, PLUGIN_TYPE_LEN);
}

//----------------------------------------------------------------------------------------------------------
// Function: GetDescription
//   Interface to get plugin description
//
// Parameters:
//   char* pDesc - Plugin Description
//
// Returns:
//   None
//----------------------------------------------------------------------------------------------------------
void GetDescription(char* pDesc)
{
   if(pDesc)
      strncpy(pDesc, PLUGINAPI_DESC, PLUGIN_DESC_LEN);
}

// Functions to learn about the reports
//----------------------------------------------------------------------------------------------------------
// Function: GetOutputTypes
//   Interface to get list of output report format types
//
// Parameters:
//   void (*pSetFormats) (const char*) - callback funtion pointer
//
// Returns:
//   None
//----------------------------------------------------------------------------------------------------------
void GetOutputTypes(void (*pSetFormats) (const char*))
{
   // Check for NULL pointer
   if(!pSetFormats)
      return;

   try
   {
      // Initialize plugin COM library
      CoInitialize(NULL);
      CRPluginWorker::ICRPluginWorkerPtr pICRPluginWorker(__uuidof(CRPluginWorker::CRPluginWorkerClass));
      if(pICRPluginWorker)
      {
         std::string strOutputTypes;
         wchar_t* pOutputTypes = pICRPluginWorker->GetReportFormatTypes();
         if(pOutputTypes)
         {
            strOutputTypes = ToUtf8(pOutputTypes);
            pSetFormats(strOutputTypes.c_str());
         }
         else
         {
            pSetFormats("");
         }
      }
      else
      {
         pSetFormats("");
      }

      // Uninitialize plugin COM library
      CoUninitialize();
   }
   catch(...)
   {
      pSetFormats("");
   }
}

//----------------------------------------------------------------------------------------------------------
// Function: GetReportList
//   Interface to get list of reports
//
// Parameters:
//   void (*pSetReports) (const char*) - Callback function pointer
//   char * error - Error string
//
// Returns:
//   None
//----------------------------------------------------------------------------------------------------------
ReportStatus GetReportList(void (*pSetReports) (const char*), char * error)
{
   ReportStatus retVal = ok;

   // Check for NULL pointer
   if(!pSetReports)
      return ReportStatus::failed;

   std::string strError;

   try
   {
      // Initialize plugin COM library
      CoInitialize(NULL);
      CRPluginWorker::ICRPluginWorkerPtr pICRPluginWorker(__uuidof(CRPluginWorker::CRPluginWorkerClass));

      if(pICRPluginWorker)
      {
         // Utf8ToWide allocates memory to store wide characters
         wchar_t* pWError = NULL;
         wchar_t* pReportList = NULL;
         std::string strReportList;

         // Initialize reporting service
         bool bInitSuccess = pICRPluginWorker->Initialize(&pWError);
         if(bInitSuccess)
         {
            pReportList = pICRPluginWorker->PopulateReportList(&pWError);
         }
         
         // Translate wide characters to character array
         if(pWError)
            strError = ToUtf8(pWError);

         if(strError.length() > 0)
         {
            retVal = failed;

            // Copy output error message
            if(strError.c_str())
               strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
         }
         else
         {
            if(pReportList)
            {
               strReportList = ToUtf8(pReportList);
               pSetReports(strReportList.c_str());
            }
            else
            {
               pSetReports("");
               retVal = failed;
            }
         }
      }
      else
      {
         strError = PLUGIN_MESG1;
         strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
      }

      // Uninitialize plugin COM library
      CoUninitialize();
   }
   catch(...)
   {
      strError = PLUGIN_MESG2;
      strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
      pSetReports("");
   }

   return retVal;
}

// Functions used to run reports
//----------------------------------------------------------------------------------------------------------
// Function: RunReport
//   Interface to run report with given reportid and type
//
// Parameters:
//   const char * reportid - Report ID
//   uint32_t reportOutputTypeId - Report format type
//   const char * project - Project to which this report belong to. This will be set as a parameter
//                          if report's only parameter is "project name"
//   char * outputfile - Rendered report file
//   char * error - Error message
//
// Returns:
//   ReportStatus - Report Status
//----------------------------------------------------------------------------------------------------------
ReportStatus RunReport(const char * reportid, uint32_t reportOutputTypeId, 
                       const char * project, char * outputfile, char * error)
{
   ReportStatus retVal = ok;
   std::string strError;

   try
   {
      // Initialize plugin COM library
      CoInitialize(NULL);
      CRPluginWorker::ICRPluginWorkerPtr pICRPluginWorker(__uuidof(CRPluginWorker::CRPluginWorkerClass));

      if(pICRPluginWorker)
      {
         wchar_t* pWOutFile = NULL;
         wchar_t* pWError = NULL;
         std::string strOutputFile;

         // Initialize reporting service
         bool bInitSuccess = pICRPluginWorker->Initialize(&pWError);
         if(bInitSuccess)
            retVal = (ReportStatus) pICRPluginWorker->RunExternalReport(reportid, reportOutputTypeId, project, &pWOutFile, &pWError);

         // Translate wide characters to character array
         if(pWError)
            strError = ToUtf8(pWError);

         if(pWOutFile)
            strOutputFile = ToUtf8(pWOutFile);

         // Copy out params
         if(strError.length() > 0)
            strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
         if(strOutputFile.length() > 0)
            strncpy(outputfile, strOutputFile.c_str(), PLUGIN_OUTFILE_LEN);
      }
      else
      {
         strError = PLUGIN_MESG1;
         strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
      }

      CoUninitialize();
   }
   catch(...)
   {
      strError = PLUGIN_MESG3;
      strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
   }

   return retVal;
}

//----------------------------------------------------------------------------------------------------------
// Function: ExecuteClientCommand
//   Interface that executes given client command
//
// Parameters:
//   const char * command - Client command in XML format
//   void (*pSetResults) (const char*) - Callback function pointer (to return results)
//   char * error - Error message
//
// Returns:
//   ReportStatus - Report Status
//----------------------------------------------------------------------------------------------------------
ReportStatus ExecuteClientCommand(const char * command, 
                                  void (*pSetResults) (const char*) , 
                                  char * error)
{
   ReportStatus retVal = ok;

   // Check for NULL pointer
   if(!pSetResults)
      return ReportStatus::failed;

   wchar_t* pWError = NULL;
   std::string strError;

   try
   {
      // Initialize plugin COM library
      CoInitialize(NULL);
      CRPluginWorker::ICRPluginWorkerPtr pICRPluginWorker(__uuidof(CRPluginWorker::CRPluginWorkerClass));

      if(pICRPluginWorker && command)
      {
         wchar_t* pWCommand = Utf8ToWide(command);
         wchar_t* pWCmdResult = NULL;
         std::string strCmdResult;

         // Initialize reporting service
         bool bInitSuccess = pICRPluginWorker->Initialize(&pWError);
         if(bInitSuccess)
         {
            pWCmdResult = pICRPluginWorker->ExecuteClientCmd(pWCommand, &pWError);
         }

         if(pWError)
            strError = ToUtf8(pWError);
   
         if(pWCmdResult)
            strCmdResult = ToUtf8(pWCmdResult);

         if(strError.length() > 0)
         {
            // copy out params
            strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
            retVal = failed;
         }
         else
         {
            pSetResults(strCmdResult.c_str());
         }

         // Deallocate memory allocated for wchar_t*
         if(pWCommand)
            delete [] pWCommand;
      }
      else
      {
         strError = PLUGIN_MESG1;
         strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);
      }

      CoUninitialize();
   }
   catch(...)
   {
      strError = PLUGIN_MESG4;
      strncpy(error, strError.c_str(), PLUGIN_ERR_LEN);

      pSetResults("");
   }

   return retVal;
}


