using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;

namespace CRPluginWorker
{
   public class CRPluginWorker
   {
      [ComVisible(true)]
      [Guid("369ABDAD-740C-40e5-96BC-C9DC19399193")]
      public interface ICRPluginWorker
      {
         bool Initialize(out String p_strError);
         String PopulateReportList(out String p_strError);
         String GetReportFormatTypes();
         Int16 RunExternalReport(String reportID, Int32 reportOutputTypeId,
                        String project, out String outputfile, out String p_strError);
         String ExecuteClientCmd(String command, out String p_strError);
      }

      [ClassInterface(ClassInterfaceType.None)]
      [Guid("C7037CD8-EA28-43af-ADC5-4BB00DD99FEC")]
      [ComVisible(true)]
      public class CRPluginWorkerClass : ICRPluginWorker
      {
         public enum ReportStatus { ok, failed, hasparams }
         public enum ReportOutputFormat { rofNotSet = 0, rofCrystalApp }

         protected ServerBase[] m_aservers = null;

         public CRPluginWorkerClass()
         {
         }

         public bool Initialize(out String p_strError)
         {
            bool fResult = false;
            p_strError = "";
            try
            {
               string strPluginPath = Directory.GetParent(System.Reflection.Assembly.GetExecutingAssembly().Location).FullName;

               Config cfg = new Config(Path.Combine(strPluginPath, "serverreports"));
               m_aservers = cfg.read(out p_strError);

               fResult = (m_aservers != null && m_aservers.Length > 0);
            }
            catch (Exception p_exception) { p_strError = "Initialization failed: " + p_exception.Message; }
            return (fResult);
         }

         [return: MarshalAs(UnmanagedType.LPWStr)]
         public String PopulateReportList(out String p_strError)
         {
            string strResult = "";
            p_strError = "";
            try
            {
               for (int i = 0; m_aservers != null && i < m_aservers.Length; ++i)
               {
                  string str = "";
                  strResult += m_aservers[i].fetchReportList(out str);
                  if (str.Length > 0) { p_strError += str; }
               }
            }
            catch (Exception p_exception) { p_strError = "PopulateReportList: " + p_exception.Message; }
            return (strResult);
         }

         [return: MarshalAs(UnmanagedType.LPWStr)]
         public String GetReportFormatTypes()
         {
            string str = "";
            try
            {
               str += SetReportFormatType("1", "Crystal App"); // This is it, for now.
            }
            catch (Exception) { str = ""; }
            return (str);
         }

         public Int16 RunExternalReport(String p_strId, Int32 p_strOutputTypeId, String p_strProject, out String p_strOutputFile, out String p_strError)
         {
            Int16 iResult = (Int16)ReportStatus.failed;
            p_strOutputFile = "";
            p_strError = "";
            try
            {
               ReportOutputFormat rof = (ReportOutputFormat)Convert.ToInt32(p_strOutputTypeId);
               if (rof == ReportOutputFormat.rofNotSet) { rof = promptForReportRunFormat(); }

               ServerBase s = getServerFromReportId(p_strId);
               if (s != null)
               {
                  string tempPath = System.IO.Path.GetTempPath();
                  p_strOutputFile = s.fetchReport(p_strId, out p_strError);
                  File.Copy(p_strOutputFile, tempPath + "\\Report.rpt", true);
                  p_strOutputFile = tempPath + "\\Report.rpt";
                  if (p_strOutputFile != null && p_strOutputFile.Length > 0)
                     iResult = (Int16)ReportStatus.ok;
                  else
                  {
                     iResult = (Int16)ReportStatus.failed;
                     p_strError = "Report '" + p_strId + "' could not be located.";
                  }
               }
            }
            catch (Exception p_exception) { p_strError = "RunExternalReport failed: " + p_exception.Message; }
            return (iResult);
         }

         [return: MarshalAs(UnmanagedType.LPWStr)]
         public String ExecuteClientCmd([MarshalAs(UnmanagedType.LPWStr)] String command, [MarshalAs(UnmanagedType.LPWStr)] out String p_strError)
         {
            p_strError = "not supported";
            return ("");
         }

         private ServerBase getServerFromReportId(string p_str)
         {
            ServerBase s = null;
            for (int i = 0; m_aservers != null && i < m_aservers.Length && s == null; ++i)
            {
               if (m_aservers[i].isValidReport(p_str)) { s = m_aservers[i]; }
            }
            return (s);
         }

         // Forms xml string for the given report id and format
         static private string SetReportFormatType(string id, string desc)
         {
            string strRepFmt = "<outputFormat>";
            strRepFmt += "<id>" + id + "</id>";
            strRepFmt += "<description>" + desc + "</description>";
            strRepFmt += "</outputFormat>";
            return strRepFmt;
         }

         private ReportOutputFormat promptForReportRunFormat()
         {
            ReportOutputFormat rof = ReportOutputFormat.rofNotSet;
            try
            {
               rof = ReportOutputFormat.rofCrystalApp; // This is it, for now.
            }
            catch (Exception) {}
            return (rof);
         }
      }
   }
}
