using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Xml;

namespace CRPluginWorker
{
   public class Config
   {
      const string c_strServerTypeNode_Local = "local";

      private const string c_strRootNode = "CRConfig";
      private const string c_strConfigFile = "crplugin.xml";

      private string m_strConfigFile = "";

      public Config(string p_strConfigFilePath) { if (p_strConfigFilePath.Length > 0) { m_strConfigFile = Path.Combine(p_strConfigFilePath, c_strConfigFile); } }

      public ServerBase[] read(out string p_strError)
      {
         ServerBase[] asb = null;
         p_strError = "";
         try
         {
            XmlDocument xml = loadConfigFile(out p_strError); // Load the config file into memory
            if (xml != null)
            {
               ArrayList list = new ArrayList();
               for(int i = 0; xml.DocumentElement != null && i < xml.DocumentElement.ChildNodes.Count; ++i)
               {
                  XmlNode n = xml.DocumentElement.ChildNodes[i];
                  if (n != null)
                  {
                     ServerBase s = null;
                     if (n.Name == c_strServerTypeNode_Local)
                        s = new ServerLocal();

                     if (s != null && s.populate(n)) { list.Add(s); }
                  }
               }

               asb = new ServerBase[list.Count];
               for (int i = 0; i < list.Count; ++i) { asb[i] = (ServerBase)list[i]; }
            }
         }
         catch (Exception p_exception) { p_strError = "Unable to read config file: " + p_exception.Message; }
         return (asb);
      }

      protected XmlDocument loadConfigFile(out string p_strError)
      {
         XmlDocument xml = null;
         p_strError = "";
         try
         {
            xml = new XmlDocument();
            xml.Load(m_strConfigFile);
         }
         catch (Exception p_exception) { p_strError = "Unable to load config file: " + p_exception.Message; xml = null; }
         return (xml);
      }
   }
}
