using System;
using System.IO;
using System.Text;
using System.Xml;

namespace CRPluginWorker
{
   public enum ServerType { stUnknown = 0, stLocal };

   // Thinking here is to eventually support Crystal Reports Server and possibly even the BO BI stuff.
   public abstract class ServerBase
   {
      protected ServerType m_st = ServerType.stUnknown;

      protected string m_strId = "";
      protected string m_strAddress = "";
      protected string m_strPort = "";

      public ServerBase() {}

      public string getId() { return (m_strId); }
      public void setId(string p_str) { m_strId = p_str; }

      public string getAddress() { return (m_strAddress); }
      public void setAddress(string p_str) { m_strAddress = (Path.IsPathRooted(p_str) ? p_str:Path.Combine(Directory.GetParent(System.Reflection.Assembly.GetExecutingAssembly().Location).FullName, Path.Combine("serverreports", p_str))); }

      public string getPort() { return (m_strPort); }
      public void setPort(string p_str) { m_strPort = p_str; }

      public abstract bool populate(XmlNode p_n);
      public abstract bool isValid();
      public abstract string fetchReportList(out string p_strError);
      public abstract string fetchReport(string p_strReportId, out string p_strError);
      public abstract bool isValidReport(string p_strReportId);

      protected string buildReportId(string p_strReport)
      {  // No XML, we only have 1024 characters;
         return (getId() + ":" + Path.GetFileName(p_strReport));
      }

      protected string decompileReportId(string p_strReport, out string p_strServerId)
      {
         string str = "";
         p_strServerId = "";
         try 
      	{
            string[] astr = p_strReport.Split(':');
            if (astr != null && astr.Length >= 2)
            {
               p_strServerId = astr[0];
               str = astr[1];
            }
      	}
      	catch (Exception) {}
         return (str);
      }

      protected string buildReportInfoObject(string p_strId, string p_strName, string p_strTitle, string p_strAddtlInfo)
      {
         string strResult = "";
         try
         {
            // Add reportinfo tag
            strResult += "<reportinfo>";

            // Append report id
            strResult += "<id>";
            strResult += p_strId;
            strResult += "</id>";

            // Append report name
            strResult += "<originalName>";
            strResult += p_strName;
            strResult += "</originalName>";

            // Append report title
            strResult += "<title>";
            strResult += p_strTitle;
            strResult += "</title>";

            // Append additional info (Path for now)
            strResult += "<additional-info>";
            strResult += p_strAddtlInfo;
            strResult += "</additional-info>";

            // Add end of reportinfo tag
            strResult += "</reportinfo>";
         }
         catch (Exception) { strResult = ""; } // Don't add a half-created string.
         return (strResult);
      }
   }
}
